local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-ItemLine", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs
local tonumber = tonumber
local abs, floor = math.abs, math.floor
local format = string.format

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent
local GetItemIcon, GetItemInfo = GetItemIcon, GetItemInfo

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local MAIN_SOURCE_FLAG_ICONS = GGL.MAIN_SOURCE_FLAG_ICONS
local SUB_SOURCE_FLAG_ICONS = GGL.SUB_SOURCE_FLAG_ICONS
local MAIN_SOURCE_FLAG = GGL.MAIN_SOURCE_FLAG

local GAP_SIZE = 2
local FIX_HEIGHT = 14

local ITEM_NUM_FORMAT = "%s x %d"

local GetMoneyText = GGL.GetMoneyText

local function Frame_OnEnter(self)
    if not self.obj.itemID and not self.obj.itemString then return end
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    GGL.GUI:SetToolTipPoint(tooltip, self.obj.ttParent, self.obj.ttPoint, 2, 0)
    if self.obj.itemString then
        tooltip:SetHyperlink(self.obj.itemString)
    else
        tooltip:SetItemByID(self.obj.itemID)
    end
    tooltip:Show()
end

local function Frame_OnLeave(self)
    _G.GameTooltip:Hide()
end

local function Frame_OnClick(self, button)
    if not self.obj.itemID and not self.obj.itemString then return end
    GGL:ItemMouseOnClick(self.obj.itemString or self.obj.itemID, self, button)
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self.itemID = nil
        self.itemString = nil
        self.sourceItemID = nil
        self.quantity = nil
        self.money = nil
        self:ResetToolTipPoint()
        self.sortValue = 0
	end,

    ["OnWidthSet"] = function(self, width)

    end,

    ["SetName"] = function(self, name)
        self.name:SetText(name)
    end,

    ["SetToolTipPoint"] = function(self, ttPoint, parent)
        self.ttPoint = ttPoint or "BOTTOM_X"
        self.ttParent = parent or self.frame
    end,

    ["ResetToolTipPoint"] = function(self)
        self.ttPoint = "BOTTOM_X"
        self.ttParent = self.frame
    end,

    ["SetItem"] = function(self, itemID, itemString, sourceItemID, quantity, moneyValue, mainSource, subSource)
        if not itemID then return end

        local itemName, itemLink = GetItemInfo(itemString or itemID)

        self.imageL:SetTexture(GetItemIcon(itemID))
        self.text1:SetFormattedText(ITEM_NUM_FORMAT, itemLink or "", tonumber(quantity) or 0)
        self.text2:SetText(GetMoneyText(moneyValue))
        -- Get source textures
        if sourceItemID and mainSource == MAIN_SOURCE_FLAG.ITEM then
            self.imageR:SetTexture(GetItemIcon(sourceItemID))
            self.imageR:Show()
        else
            mainSource = ( mainSource and MAIN_SOURCE_FLAG_ICONS[mainSource] ) and MAIN_SOURCE_FLAG_ICONS[mainSource] or nil
            subSource = ( subSource and SUB_SOURCE_FLAG_ICONS[subSource] ) and SUB_SOURCE_FLAG_ICONS[subSource] or nil
            if mainSource or subSource then
                self.imageR:SetTexture(subSource or mainSource)
                self.imageR:Show()
            else
                self.imageR:Hide()
            end
        end

        self.itemID = itemID
        self.itemString = itemString
        self.sourceItemID = sourceItemID
        self.quantity = quantity
        self.money = moneyValue
    end,

    ["SetSortValue"] = function(self, value)
        self.sortValue = value or 0
    end,

    ["GetSortValue"] = function(self)
        return self.sortValue
    end,
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
    local widget = {}
    widget.type  = Type

    local frame = CreateFrame("Button", nil, UIParent)
    frame:SetHeight(FIX_HEIGHT)
    frame:SetScript("OnEnter", Frame_OnEnter)
    frame:SetScript("OnLeave", Frame_OnLeave)
    frame:SetScript("OnClick", Frame_OnClick)
    frame.obj = widget
    widget.frame = frame

    local imageL = frame:CreateTexture(nil, "BACKGROUND")
    imageL:SetPoint("LEFT", frame, "LEFT", 0, 0)
    imageL:SetWidth(FIX_HEIGHT-2)
    imageL:SetHeight(FIX_HEIGHT-2)
    widget.imageL = imageL

    local imageR = frame:CreateTexture(nil, "BACKGROUND")
    imageR:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
    imageR:Hide()
    imageR:SetWidth(FIX_HEIGHT-2)
    imageR:SetHeight(FIX_HEIGHT-2)
    widget.imageR = imageR

    local text1 = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    text1:SetPoint("LEFT", imageL, "RIGHT", GAP_SIZE, 0)
    text1:SetPoint("RIGHT", imageR, "LEFT", -1, 0)
    text1:SetJustifyH("LEFT")
    text1:SetJustifyV("MIDDLE")
    text1:SetWordWrap(false)
    text1:SetHeight(FIX_HEIGHT)
    widget.text1 = text1

    local text2 = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    text2:SetAllPoints(text1)
    text2:SetJustifyH("RIGHT")
    text2:SetJustifyV("MIDDLE")
    text2:SetWordWrap(false)
    widget.text2 = text2

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
