local AddonName, GGL = ...

local AceGUI = LibStub("AceGUI-3.0")
local Templates = GGL.Templates
local L = GGL.Locale

-- lua
local pairs, assert, type = pairs, assert, type
local tblsort, tblremove = table.sort, table.remove
local math_max = math.max
local xpcall = xpcall
local time, date = time, date
local format = string.format

-- wow

local MainWindow = {}
GGL.GUI.MainWindow = MainWindow

local GetMoneyText = GGL.GetMoneyText
local GetMoneyTextWithIcons = GGL.GetMoneyTextWithIcons
local CreateLine = GGL.GUI.CreateLine

local function OnMapChange(sender, mapID, mapName, mapTypeFlag, difficultyID)
    if not mapID then return end
    local frame = MainWindow.frame
    frame:UpdateZone(mapName, mapTypeFlag)
    if frame.ZoneFilter and not frame.ZoneFilter:IsValidZone(mapID) then
        frame.contentContainer.zoneWarningFrame.warningText = format(L["Zone '%s' is forbidden."], mapName or "")
        if not frame.contentContainer.zoneWarningFrame:IsShown() then
            frame.contentContainer.zoneWarningFrame:Show()
            frame.contentContainer.zoneText:SetPoint("RIGHT", frame.contentContainer.zoneWarningFrame, "LEFT", -2, 0)
        end
    elseif frame.contentContainer.zoneWarningFrame:IsShown() then
        frame.contentContainer.zoneWarningFrame:Hide()
        frame.contentContainer.zoneText:SetPoint("RIGHT", frame.contentContainer.zoneContainer)
    end
end

local function OnInstancesResetUpdate(sender, instanceLocks, nextReleaseStartTime, nextReleaseEndTime)
    MainWindow.frame:UpdateReset(instanceLocks, nextReleaseStartTime, nextReleaseEndTime)
end

function MainWindow:Show()
    self:CreateFrame()

    self.frame:Show()

    OnMapChange(nil, GGL:GetZoneData())

    return self.frame
end

-- TODO: remove
function MainWindow:LoadSession(session, options)
    self:CreateFrame()

    self.frame:Show()

    session = session or Templates:New("Session", GGL.sessionDb.Session)

    if options and not session:DataIsSet() then
        session:SetData((options.duration or 5)*60, options.set or "Base")
    end
    session:AddIntoMainFrame(true)

    session:Start()
end

-- ##################
-- Window Creation
-- ##################
local WIDTH_V, HEIGHT_V = 370, 212
local WIDTH_M, HEIGHT_M = 150, 129
local MAX_ITEM_ENTRYS = 50
local TIME_FORMAT = "%H:%M"
local TT_INFO_ENTRY = "|cFFCFCFCF%s:|r %s"

local BACKDROP = {
    bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
    edgeSize = 2,
    insets = { left = 1, right = 1, top = 1, bottom = 1 }
}

local FRAME_FUNCTIONS = {
    ["Clear"] = function(self)
        self.itemContainer:ReleaseChildren()

        -- reset gold
        self:SetItemValueGold(0)
        self:SetRawGold(0)
        self:SetTotalGold(0)
        self:SetKillCount(0)
        self:SetEndlessTimer()
        self:SetSession()
    end,

    ["SetSession"] = function(self, session)
        self.session = session
        if not session then
            self.contentContainer.pauseSessionButton:SetText(L["Resume Session"])
        end
    end,

    ["SetMaxItemEntrys"] = function(self, num)
        self.maxItemEntrys = num
        self.itemContainer:ReleaseChildren()
    end,

    ["AddItem"] = function(self, itemID, itemString, sourceItemID, quantity, moneyValue, mainSource, subSource)
        if not itemID then return end
        local children = self.itemContainer.children
        local itemRow

        -- clear first item if we are over cap
        if #children+1 > self.maxItemEntrys then
            local content = self.itemContainer.content
            itemRow = tblremove(children, 1)
            local frame = itemRow.frame

            frame:ClearAllPoints()
            children[#children].frame:SetPoint("BOTTOMLEFT", frame, "TOPLEFT")
            frame:SetPoint("BOTTOMLEFT", content)
            frame:SetPoint("RIGHT", content)

            children[#children+1] = itemRow
        end

        if not itemRow then
            itemRow = AceGUI:Create("GoldGoblinLootGUIWidget-ItemLine")
            itemRow:SetFullWidth(true)
            self.itemContainer:AddChild(itemRow)
            itemRow:SetToolTipPoint("BOTTOM_X", self.frame)
        end

        itemRow:SetItem(itemID, itemString, sourceItemID, quantity, moneyValue, mainSource, subSource)
        self.itemContainer:SetScroll(1000)

        self.itemContainer.frame:Show()
    end,

    ["SetItemValueGold"] = function(self, copper, add)
        self.itemValueGoldCount = add and (self.itemValueGoldCount or 0) + copper or copper
        self.contentContainer.itemValue:SetText(GetMoneyText(self.itemValueGoldCount))
        self:SetTotalGold((self.rawGoldCount or 0) + (self.itemValueGoldCount or 0))
    end,

    ["SetRawGold"] = function(self, copper, add)
        if not copper or copper < 0 then return end
        self.rawGoldCount = add and ((self.rawGoldCount or 0) + copper) or copper
        self.contentContainer.rawGold:SetText(GetMoneyText(self.rawGoldCount))
        self:SetTotalGold((self.rawGoldCount or 0) + (self.itemValueGoldCount or 0))
    end,

    ["SetTotalGold"] = function(self, copper, add)
        self.sumGoldCount = add and (self.sumGoldCount or 0) + copper or copper
        self.contentContainer.goldSum:SetText(GetMoneyTextWithIcons(self.sumGoldCount, self.contentContainer.goldSum.fontHeight, true))
    end,

    ["SetKillCount"] = function(self, count, add)
        self.rawKillCount = add and (self.rawKillCount or 0) + count or count
        self.contentContainer.killCount:SetText(self.rawKillCount)
    end,

    ["UpdateReset"] = function(self, instanceLocks, nextReleaseStartTime, nextReleaseEndTime)
        local frame = self.contentContainer
        if not instanceLocks or not nextReleaseEndTime then
            frame.instanceResetButton.text:SetFormattedText(L["Reset Instances (%d/9)"], 0)
            frame.instanceResetTimer:StopTimer()
            frame.instanceResetButton.instanceLocks = nil
        else
            frame.instanceResetButton.text:SetFormattedText(L["Reset Instances (%d/9)"], #instanceLocks)
            frame.instanceResetTimer:SetTime(nextReleaseStartTime, nextReleaseEndTime)
            frame.instanceResetTimer:StartTimer()
            frame.instanceResetButton.instanceLocks = instanceLocks
        end
        -- refresh tooltip
        if frame.instanceResetButton.isOnEnter then
            frame.instanceResetButton:Fire("OnEnter")
        end
    end,

    ["UpdateZone"] = function(self, mapName, mapTypeFlag)
        local frame = self.contentContainer

        if mapTypeFlag == GGL.MAP_TYPE_FLAGS.RAID then
            frame.zoneIcon:SetAtlas("Raid")
        elseif mapTypeFlag == GGL.MAP_TYPE_FLAGS.DUNGEON then
            frame.zoneIcon:SetAtlas("Dungeon")
        elseif mapTypeFlag == GGL.MAP_TYPE_FLAGS.ISLAND_EXPEDITION then
            frame.zoneIcon:SetAtlas("poi-islands-table")
        else
            frame.zoneIcon:SetAtlas("Vehicle-Carriage")
        end

        frame.zoneText:SetText(mapName)
    end,

    ["RefreshMap"] = function(self)
        OnMapChange(nil, GGL:GetZoneData())
    end,

    ["SetEndlessTimer"] = function(self, timeElapsed, noButton)
        if timeElapsed then
            self.contentContainer.timer:SetEndless(timeElapsed)
        end

        if timeElapsed and not noButton then
            self.contentContainer.pauseSessionButton.frame:Show()
            self.contentContainer.pauseSessionButton:SetText(L["Pause Session"])
        else
            self.contentContainer.pauseSessionButton.frame:Hide()
        end
    end,

    ["RefreshDBSettings"] = function(self)
        if GGL.db.Settings.ShowDestroyGrayButton then
            self.contentContainer.destroyGrayButton.frame:Show()
        else
            self.contentContainer.destroyGrayButton.frame:Hide()
        end
    end
}

local function ZoneWarning_OnEnter(self)
    -- contentContainer.zoneWarningFrame
    if self.warningText then
        local tooltip = _G.GameTooltip
        tooltip:SetOwner(self, "ANCHOR_NONE")
        tooltip:ClearAllPoints()
        tooltip:SetPoint("BOTTOMLEFT", self.ttFrame, "BOTTOMRIGHT", 2, 0)
        tooltip:AddLine(L["Warning:"])
        tooltip:AddLine(self.warningText)
        tooltip:Show()
    end
end

local function ResetLockout_OnEnter(self)
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    GGL.GUI:SetToolTipPoint(tooltip, self.ttFrame, "BOTTOM_X")
    tooltip:AddLine(format(L["Instance lockout: %d/9"], self.instanceLocks and #self.instanceLocks or 0))
    if self.instanceLocks then
        for i = 1, #self.instanceLocks do
            tooltip:AddDoubleLine(date(TIME_FORMAT, self.instanceLocks[i].endTime), self.instanceLocks[i].zone)
        end
    end
    tooltip:Show()
    self.isOnEnter = true
end

local function ResetLockout_OnLeave(self)
    _G.GameTooltip:Hide()
    self.isOnEnter = false
end

local function DestroyGray_OnEnter(self)
    if not self.itemValue or self.itemValue <= 0 then return end
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    GGL.GUI:SetToolTipPoint(tooltip, self.ttFrame, "BOTTOM_X")
    tooltip:AddLine(format(L["Destroy %d gray items"], self.counter or 0))
    tooltip:AddDoubleLine(L["Vendor Value"], GetMoneyText(self.itemValue or 0), 1, 1, 1, 1, 1, 1)
    tooltip:Show()
    self.isOnEnter = true
end

local function ResetLockout_OnClick(self, button)
    ResetInstances()
end

local function PauseSession_OnClick(self, event, mb)
    if self.obj.session then
        local sessionType = self.obj.session.sessionType
        GGL.Session:StopSession(sessionType)
        GGL.Session:Options_Update(sessionType)
    else
        GGL.Session:StartSession("A")
        GGL.Session:Options_Update("A")
    end
end

local function DestroyGrayButton_OnEvent(self, event)
    if self.obj then self = self.obj end
    if self.pause then return end
    -- BAG_UPDATE:
    local counter = 0
    local itemValue = 0

    for bag = 0, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots(bag) do
            local icon, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(bag, slot)
			if icon and quality == 0 then
                counter = counter + 1
                local _, _, _, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(itemLink)
                itemValue = itemValue + ((itemSellPrice or 0)*(itemCount or 0))
			end
        end
    end

    if counter ~= self.counter then
        self.text:SetFormattedText(L["Destroy %d gray items"], counter)
        self.counter = counter
    end

    self.itemValue = itemValue
    -- update tooltip
    if self.isOnEnter then
        ResetLockout_OnLeave(self)
        DestroyGray_OnEnter(self)
    end
    return counter
end

local function DestroyGrayButton_OnClick(self, button)
    self.pause = true
    GGL:DestroyGrayBagItems()
    self.pause = false
    DestroyGrayButton_OnEvent(self, "BAG_UPDATE")
end

local function DestroyGrayButton_OnMouseWheel(self, delta)
    -- -1 = down, 1 = up
    if delta == -1 then
        DestroyGrayButton_OnClick(self)
    end
end

function MainWindow:CreateFrame()
    if not self.frame then
        AceGUI = LibStub("AceGUI-3.0")

        local window = AceGUI:Create("Window")
        --window:SetLayout("Flow")
        window:SetTitle("GoldGoblinLoot")

        local frame = window.frame
        local content = window.content

        -- some frame settings
        frame:SetMinResize(WIDTH_V,HEIGHT_V)
        frame:SetMaxResize(WIDTH_V+WIDTH_M,HEIGHT_V+HEIGHT_M)
        frame:SetClampedToScreen(true)
        frame:SetToplevel(false)
        frame:SetFrameStrata("MEDIUM")
        --window settings
        window:SetWidth(WIDTH_V)
        window:SetHeight(HEIGHT_V)
        window.maxItemEntrys = MAX_ITEM_ENTRYS
        -- remove resizer lines
        window.line1:Hide()
        window.line2:Hide()

        --itemContainerBG
        local itemContainerBG = GGL.GUI:CreateBorderedFrame(frame)

        -- itemContainer
        local itemContainer = AceGUI:Create("ScrollFrame")
        itemContainer.frame:SetParent(content) -- fix for a bug with skinning
        itemContainer.frame:SetPoint("TOPLEFT", content)
        itemContainer:SetLayout("GGL-Down2Up")
        --itemContainer
        window.itemContainer = itemContainer

        itemContainerBG:SetPoint("TOPLEFT", itemContainer.scrollframe, -5, 5)
        itemContainerBG:SetPoint("BOTTOMRIGHT", itemContainer.scrollframe, 5, -5)

        -- contentContainer
        local contentContainerF = GGL.GUI:CreateBorderedFrame(frame)

        local contentContainer = CreateFrame("Frame", nil, contentContainerF)
        contentContainer:SetPoint("BOTTOMLEFT", content, 0, 0)
        contentContainer:SetPoint("BOTTOMRIGHT", content, 0, 0)
        contentContainer:SetHeight(118)
        window.contentContainer = contentContainer

        contentContainerF:SetPoint("TOPLEFT", contentContainer, -5, 5)
        contentContainerF:SetPoint("BOTTOMRIGHT", contentContainer, 5, -5)

        itemContainer.frame:SetPoint("BOTTOMRIGHT", contentContainer, "TOPRIGHT", 0, 8)

        -- ## timer
        local timerContainer = GGL.GUI:CreateBoxFrame(contentContainer)
        timerContainer:SetPoint("TOPRIGHT", contentContainer, "TOPRIGHT", -3, -3)
        timerContainer:SetSize(100, 27)
        local timerContainerSize = 100

        local timer = AceGUI:Create("GoldGoblinLootGUIWidget-Timer")
        timer.frame:SetParent(contentContainer)
        timer:SetPoint("CENTER", timerContainer, 0, 0)
        timer:SetSize(20)
        timer:SetDuration(0)
        contentContainer.timer = timer
        -- ## timer end

        -- ############
        -- LEFT Text
        -- ############

        -- ## zone
        local zoneContainer = CreateFrame("FRAME", nil, contentContainer)
        zoneContainer:SetPoint("LEFT", contentContainer, 2, 0)
        zoneContainer:SetPoint("TOPRIGHT", timerContainer, "TOPLEFT", -2, 0)
        zoneContainer:SetPoint("BOTTOMRIGHT", timerContainer, "BOTTOMLEFT", -2, 0)
        contentContainer.zoneContainer = zoneContainer

        local zoneIcon = zoneContainer:CreateTexture(nil, "BACKGROUND")
        zoneIcon:SetPoint("LEFT", zoneContainer)
        zoneIcon:SetSize(zoneContainer:GetHeight()-2, zoneContainer:GetHeight()-2)
        zoneIcon:SetAtlas("Vehicle-Carriage")
        contentContainer.zoneIcon = zoneIcon

        local zoneWarningFrame = CreateFrame("Frame", nil, contentContainer)
        zoneWarningFrame:SetPoint("RIGHT", zoneContainer, 0, 0)
        zoneWarningFrame:SetSize(zoneIcon:GetHeight(), zoneIcon:GetHeight())
        zoneWarningFrame:SetScript("OnEnter", ZoneWarning_OnEnter)
        zoneWarningFrame:SetScript("OnLeave", ResetLockout_OnLeave)
        zoneWarningFrame.ttFrame = frame
        zoneWarningFrame:Hide()
        contentContainer.zoneWarningFrame = zoneWarningFrame

        local zoneWarning = zoneWarningFrame:CreateTexture(nil, "BACKGROUND")
        zoneWarning:SetAllPoints(zoneWarningFrame)
        zoneWarning:SetAtlas(GGL.ATLAS_TEXTURE.WARNING)
        zoneWarningFrame.texture = zoneWarning

        local zoneText = zoneContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
        zoneText:SetPoint("LEFT", zoneIcon, "RIGHT", 2, 0)
        zoneText:SetPoint("RIGHT", zoneContainer)
        zoneText:SetPoint("TOP", zoneContainer)
        zoneText:SetPoint("BOTTOM", zoneContainer)
        zoneText:SetJustifyH("LEFT")
        zoneText:SetJustifyV("CENTER")
        zoneText:SetText("")
        contentContainer.zoneText = zoneText
        -- ## zone end

        local gfhFontName, gfhFontHeight, gfhFontFlags = GameFontHighlight:GetFont()

        -- ## gold sum
        gfhFontHeight = 13
        local goldSumContainer = CreateFrame("FRAME", nil, contentContainer)
        goldSumContainer:SetPoint("TOPLEFT", zoneContainer, "BOTTOMLEFT", 0, -2)
        goldSumContainer:SetPoint("RIGHT", zoneContainer, 0, 0)
        goldSumContainer:SetHeight(20)

        local goldSum = goldSumContainer:CreateFontString(nil, "OVERLAY")
        goldSum:SetFont(gfhFontName, gfhFontHeight, gfhFontFlags)
        goldSum:SetPoint("RIGHT", goldSumContainer, -5, 0)
        goldSum:SetJustifyH("RIGHT")
        goldSum:SetJustifyV("CENTER")
        goldSum:SetHeight(goldSumContainer:GetHeight())
        goldSum.fontHeight = gfhFontHeight
        goldSum:SetHeight(gfhFontHeight)
        goldSum:SetText(GetMoneyTextWithIcons(0, gfhFontHeight, true))
        contentContainer.goldSum = goldSum

        local goldSumDesc = goldSumContainer:CreateFontString(nil, "OVERLAY")
        goldSumDesc:SetFont(gfhFontName, gfhFontHeight, gfhFontFlags)
        goldSumDesc:SetPoint("LEFT", goldSumContainer, 0, 0)
        goldSumDesc:SetPoint("RIGHT", goldSum, "LEFT", 0, 0)
        goldSumDesc:SetJustifyH("LEFT")
        goldSumDesc:SetJustifyV("CENTER")
        goldSumDesc:SetText(L["Total Gold"])
        goldSumDesc:SetTextColor(1, 0.84313725490196, 0.03921568627451)
        goldSumDesc:SetHeight(goldSumContainer:GetHeight())
        contentContainer.goldSumDesc = goldSumDesc
        -- ## gold sum end

        -- ## gold item value
        gfhFontHeight = 11
        local itemValueContainer = CreateFrame("FRAME", nil, contentContainer)
        itemValueContainer:SetPoint("TOPLEFT", goldSumContainer, "BOTTOMLEFT", 0, 0)
        itemValueContainer:SetPoint("RIGHT", zoneContainer, 0, 0)
        itemValueContainer:SetHeight(gfhFontHeight + 2)

        local itemValueDesc = itemValueContainer:CreateFontString(nil, "OVERLAY")
        itemValueDesc:SetFont(gfhFontName, gfhFontHeight, gfhFontFlags)
        itemValueDesc:SetPoint("LEFT", itemValueContainer, 0, 0)
        itemValueDesc:SetJustifyH("LEFT")
        itemValueDesc:SetJustifyV("CENTER")
        itemValueDesc:SetText(L["Item value"])
        itemValueDesc:SetTextColor(1, 1, 1)
        itemValueDesc:SetHeight(itemValueContainer:GetHeight())
        contentContainer.itemValueDesc = itemValueDesc

        local itemValue = itemValueContainer:CreateFontString(nil, "OVERLAY")
        itemValue:SetFont(itemValueDesc:GetFont())
        itemValue:SetPoint("LEFT", itemValueDesc, "RIGHT", 0, 0)
        itemValue:SetPoint("RIGHT", itemValueContainer, -5, 0)
        itemValue:SetJustifyH("RIGHT")
        itemValue:SetJustifyV("CENTER")
        itemValue:SetTextColor(1, 1, 1)
        itemValue.fontHeight = gfhFontHeight
        itemValue:SetText(GetMoneyText(0))
        contentContainer.itemValue = itemValue
        -- ## gold item value end

        -- ## raw gold
        --gfhFontHeight = 11
        local rawGoldContainer = CreateFrame("FRAME", nil, contentContainer)
        rawGoldContainer:SetPoint("TOPLEFT", itemValueContainer, "BOTTOMLEFT", 0, -1)
        rawGoldContainer:SetPoint("RIGHT", zoneContainer, 0, 0)
        rawGoldContainer:SetHeight(gfhFontHeight + 2)

        local rawGoldDesc = rawGoldContainer:CreateFontString(nil, "OVERLAY")
        rawGoldDesc:SetFont(gfhFontName, gfhFontHeight, gfhFontFlags)
        rawGoldDesc:SetPoint("LEFT", rawGoldContainer, 0, 0)
        rawGoldDesc:SetJustifyH("LEFT")
        rawGoldDesc:SetJustifyV("CENTER")
        rawGoldDesc:SetText(L["Raw gold"])
        rawGoldDesc:SetTextColor(1, 1, 1)
        rawGoldDesc:SetHeight(rawGoldContainer:GetHeight())
        contentContainer.rawGoldDesc = rawGoldDesc

        local rawGold = rawGoldContainer:CreateFontString(nil, "OVERLAY")
        rawGold:SetFont(rawGoldDesc:GetFont())
        rawGold:SetPoint("LEFT", rawGoldDesc, "RIGHT", 0, 0)
        rawGold:SetPoint("RIGHT", rawGoldContainer, -5, 0)
        rawGold:SetJustifyH("RIGHT")
        rawGold:SetJustifyV("CENTER")
        rawGold:SetTextColor(1, 1, 1)
        rawGold.fontHeight = gfhFontHeight
        rawGold:SetText(GetMoneyText(0))
        contentContainer.rawGold = rawGold
        -- ## raw gold end

        -- ## kill count
        --gfhFontHeight = 11
        local killCountContainer = CreateFrame("FRAME", nil, contentContainer)
        killCountContainer:SetPoint("TOPLEFT", rawGoldContainer, "BOTTOMLEFT", 0, -1)
        killCountContainer:SetPoint("RIGHT", zoneContainer, 0, 0)
        killCountContainer:SetHeight(gfhFontHeight + 2)

        local killCountDesc = killCountContainer:CreateFontString(nil, "OVERLAY")
        killCountDesc:SetFont(gfhFontName, gfhFontHeight, gfhFontFlags)
        killCountDesc:SetPoint("LEFT", killCountContainer, 0, 0)
        killCountDesc:SetJustifyH("LEFT")
        killCountDesc:SetJustifyV("CENTER")
        killCountDesc:SetText(L["Kills:"])
        killCountDesc:SetTextColor(1, 1, 1)
        killCountDesc:SetHeight(killCountContainer:GetHeight())
        contentContainer.killCountDesc = killCountDesc

        local killCount = killCountContainer:CreateFontString(nil, "OVERLAY")
        killCount:SetFont(killCountDesc:GetFont())
        killCount:SetPoint("LEFT", killCountDesc, "RIGHT", 0, 0)
        killCount:SetPoint("RIGHT", killCountContainer, -5, 0)
        killCount:SetJustifyH("RIGHT")
        killCount:SetJustifyV("CENTER")
        killCount:SetTextColor(1, 1, 1)
        killCount.fontHeight = gfhFontHeight
        killCount:SetText(0)
        contentContainer.killCount = killCount
        -- ## kill count end


        -- ## instance Reset
        local instanceResetContainer = CreateFrame("FRAME", nil, contentContainer)
        instanceResetContainer:SetPoint("TOPLEFT", killCountContainer, "BOTTOMLEFT", 0, -3)
        instanceResetContainer:SetPoint("RIGHT", zoneContainer, 0, 0)
        instanceResetContainer:SetHeight(gfhFontHeight + 2)

        local instanceResetTimer = AceGUI:Create("GoldGoblinLootGUIWidget-Timer")
        instanceResetTimer.frame:SetParent(contentContainer)
        instanceResetTimer:SetPoint("RIGHT", instanceResetContainer, -4, 0)
        instanceResetTimer:SetDuration(0)
        instanceResetTimer:SetSize("small")
        instanceResetTimer.frame:SetHeight(instanceResetContainer:GetHeight())
        contentContainer.instanceResetTimer = instanceResetTimer

        local instanceResetDesc = instanceResetContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
        instanceResetDesc:SetFont(gfhFontName, gfhFontHeight, gfhFontFlags)
        instanceResetDesc:SetPoint("LEFT", instanceResetContainer, 0, 0)
        instanceResetDesc:SetPoint("RIGHT", instanceResetTimer.frame, "LEFT", 0, 0)
        instanceResetDesc:SetJustifyH("LEFT")
        instanceResetDesc:SetJustifyV("CENTER")
        instanceResetDesc:SetText(L["Next Instance lockout release:"])
        instanceResetDesc:SetHeight(instanceResetContainer:GetHeight())
        instanceResetDesc:SetTextColor(1, 1, 1)
        contentContainer.instanceResetDesc = instanceResetDesc
        -- ## instance Reset end


        -- ############
        -- RIGHT Buttons
        -- ############

        local pauseSessionButton = AceGUI:Create("Button")
        pauseSessionButton.frame:SetParent(contentContainer)
        pauseSessionButton.frame:SetPoint("TOP", timerContainer, "BOTTOM", 0, -2)
        pauseSessionButton.frame:SetPoint("LEFT", timerContainer, 0, 0)
        pauseSessionButton.frame:SetPoint("RIGHT", timerContainer, 0, 0)
        pauseSessionButton.frame:Show()
        pauseSessionButton.text:ClearAllPoints()
        pauseSessionButton.text:SetPoint("TOPLEFT", 2, -1)
        pauseSessionButton.text:SetPoint("BOTTOMRIGHT", -2, 1)
        pauseSessionButton:SetText(L["Resume Session"])
        pauseSessionButton.ttFrame = frame
        pauseSessionButton:SetCallback("OnClick", PauseSession_OnClick)
        pauseSessionButton.obj = window
        contentContainer.pauseSessionButton = pauseSessionButton
        timerContainerSize = math_max(pauseSessionButton.text:GetUnboundedStringWidth() + 10, timerContainerSize)
        pauseSessionButton.frame:Hide()

        local instanceResetButton = AceGUI:Create("Button")
        instanceResetButton.frame:SetParent(contentContainer)
        instanceResetButton.frame:SetPoint("BOTTOMRIGHT", contentContainer, 0, 2)
        instanceResetButton.frame:SetPoint("LEFT", timerContainer, 0, 0)
        instanceResetButton.frame:SetPoint("RIGHT", timerContainer, 0, 0)
        instanceResetButton.frame:Show()
        instanceResetButton.text:ClearAllPoints()
        instanceResetButton.text:SetPoint("TOPLEFT", 2, -1)
        instanceResetButton.text:SetPoint("BOTTOMRIGHT", -2, 1)
        instanceResetButton:SetText(format(L["Reset Instances (%d/9)"], 0))
        instanceResetButton.ttFrame = frame
        instanceResetButton:SetCallback("OnClick", ResetLockout_OnClick)
        instanceResetButton:SetCallback("OnEnter", ResetLockout_OnEnter)
        instanceResetButton:SetCallback("OnLeave", ResetLockout_OnLeave)
        contentContainer.instanceResetButton = instanceResetButton
        timerContainerSize = math_max(instanceResetButton.text:GetUnboundedStringWidth() + 10, timerContainerSize)


        local destroyGrayButton = AceGUI:Create("Button")
        destroyGrayButton.frame:SetParent(contentContainer)
        destroyGrayButton.frame:SetPoint("BOTTOM", instanceResetButton.frame, "TOP", 0, 2)
        destroyGrayButton.frame:SetPoint("LEFT", timerContainer, 0, 0)
        destroyGrayButton.frame:SetPoint("RIGHT", timerContainer, 0, 0)
        destroyGrayButton.frame:Show()
        destroyGrayButton.text:ClearAllPoints()
        destroyGrayButton.text:SetPoint("TOPLEFT", 2, -1)
        destroyGrayButton.text:SetPoint("BOTTOMRIGHT", -2, 1)
        destroyGrayButton:SetText(format(L["Destroy %d gray items"], 999))
        destroyGrayButton:SetCallback("OnClick", DestroyGrayButton_OnClick)
        destroyGrayButton.frame:SetScript("OnMouseWheel", DestroyGrayButton_OnMouseWheel)
        destroyGrayButton.ttFrame = frame
        destroyGrayButton.frame.obj = destroyGrayButton
        destroyGrayButton.frame:RegisterEvent("BAG_UPDATE")
        destroyGrayButton.frame:SetScript("OnEvent", DestroyGrayButton_OnEvent)
        destroyGrayButton:SetCallback("OnEnter", DestroyGray_OnEnter)
        destroyGrayButton:SetCallback("OnLeave", ResetLockout_OnLeave)
        contentContainer.destroyGrayButton = destroyGrayButton
        timerContainerSize = math_max(destroyGrayButton.text:GetUnboundedStringWidth() + 10, timerContainerSize)

        -- set size of timer
        timerContainer:SetWidth(timerContainerSize)

        -- register functions
        for k,v in pairs(FRAME_FUNCTIONS) do
            window[k] = v
        end
        FRAME_FUNCTIONS = nil
        self.frame = window

        GGL:RegisterCallbackEvent("GGL_OnMapChange", OnMapChange)
        GGL:RegisterCallbackEvent("GGL_OnInstancesResetUpdate", OnInstancesResetUpdate)

        GGL:ZONE_CHANGED_NEW_AREA()
        GGL:InstanceResets_Update()
        DestroyGrayButton_OnEvent(destroyGrayButton, "BAG_UPDATE")

        GGL.GUI:AddFrameControl("MainWindow", frame, true, true)

        window:RefreshDBSettings()
        window:Hide()
    end
    return self.frame
end

function MainWindow:AddIntoMiniMapButton()
    GGL:RegisterMMFunction(self, self.Show)
end

function MainWindow:RemoveFromMiniMapButton()
    GGL:UnregisterMMFunction(self.Show)
end